#!/bin/sh
PROG=`basename $0`

DEFPMAX=10
DEFTIMEOUT=5
DEFSLEEPTIME=1

ARGS=
PMAX=$DEFPMAX
TIMEOUT=$DEFTIMEOUT
USEPORT=
DOICMP=Y
DODNS=Y
CONCURRENT=N
LOOP=N
SLEEPTIME=$DEFSLEEPTIME
USAGE=0
while :
do
	case "$1" in
	-i)	# do not do ping
		DOICMP=N
		ARGS="$ARGS $1"
		;;
	-n)	# do not do DNS lookup
		DODNS=N
		ARGS="$ARGS $1"
		;;
	-p)	# give PORTS to use (on hosts that do not give ports to lookup)
		USEPORT=$2
		ARGS="$ARGS $1 $2"
		shift
		;;
	-p*)
		USEPORT=`echo "$1" | sed 's/^-.//'`
		ARGS="$ARGS $1"
		;;
	-t)	# timeout for PING and curl
		TIMEOUT=$2
		ARGS="$ARGS $1 $2"
		shift
		;;
	-t*)
		TIMEOUT=`echo "$1" | sed 's/^-.//'`
		ARGS="$ARGS $1"
		;;
	-c)	# concurrent
		CONCURRENT=Y
		ARGS="$ARGS $1"
		;;
	-m)	# max parallel processing
		CONCURRENT=Y
		PMAX=$2
		ARGS="$ARGS $1 $2"
		shift
		;;
	-m*)
		CONCURRENT=Y
		PMAX=`echo "$1" | sed 's/^-.//'`
		ARGS="$ARGS $1"
		;;
	-l)	# loop
		LOOP=Y
		# do not remember this arg
		;;
	-s)	# sleep time when looping
		LOOP=Y
		SLEEPTIME=$2
		# do not remember this arg
		shift
		;;
	-s*)	# sleep time when looping
		LOOP=Y
		SLEEPTIME=`echo "$1" | sed 's/^-.//'`
		# do not remember this arg
		shift
		;;
	-*)
		echo "$PROG: unknown option $1" >&2
		USAGE=1
		;;
	*)
		break
		;;
	esac
	shift
done

# check -p ports and all HOST:PORT values for PORT - lookups all work
for P in `echo $USEPORT | sed 's/,/ /g'` `for HP in $*;do echo "$HP" | grep : | sed 's/.*://;s/,/ /g';done`
do
	N=`expr $P + 1 2>/dev/null`
	case "$N" in
	'')
		LPORT=`grep "^$P[ 	]*[0-9]*/tcp" /etc/services 2>/dev/null | sed 's,/tcp.*,,;s,.*[ 	],,'`
		case "$LPORT" in
		''|*\ *)
			echo "$PROG: port unknown: $P" >&2
			USAGE=1
			;;
		esac
		# it would be nice to remember all the lookups, but this SHELL - too bad
		;;
	esac
done

# make sure timeout is numeric
TTIMEOUT=`expr $TIMEOUT + 1 2>/dev/null`
case "$TTIMEOUT" in
'')
	echo "$PROG: timeout non numeric: $TIMEOUT" >&2
	USAGE=1
	;;
esac

# make sure pmax is numeric
CHKPMAX=`expr $PMAX + 1 2>/dev/null`
case "$CHKPMAX" in
'')
	echo "$PROG: -m non numeric: $PMAX" >&2
	USAGE=1
	;;
esac

# make sure sleep time is numeric
CHKSLEEPTIME=`expr $SLEEPTIME + 1 2>/dev/null`
case "$CHKSLEEPTIME" in
'')
	echo "$PROG: -s non numeric: $SLEEPTIME" >&2
	USAGE=1
	;;
esac

case "$#" in
0)
	echo "$PROG: no hostnames given" >&2
	USAGE=1
	;;
esac

case "$USAGE" in
1)
	echo "Usage: $PROG [-i] [-n] [-p PORT,...] [-c] [-m MAXPROCS] [-l] [-s SLEEP] [-t TIMEOUT] <HOST[:PORT[,PORT...]]>..." >&2
	echo "	-i	do not send icmp (no ping)" >&2
	echo "	-n	do not do DNS resolution" >&2
	echo "	-p PORT	use PORT(s) on everyhost (that does not specify any ports)" >&2
	echo "	-c	concurrent - if given multiple hostnames" >&2
	echo "	-m MAX	max number of parallel processes (default $DEFPMAX) - activates -c" >&2
	echo "	-l	loop - printing out, date if anything changes" >&2
	echo "	-s SECS	sleep time (in seconds) for loop (default $DEFSLEEPTIME) - activates -l" >&2
	echo "	-t SECS	timeout (in seconds) for ping and connect (default $DEFTIMEOUT)" >&2
	exit 1
	;;
esac

# Search for commands we might need - unless we inherited them from parent process
if [ -z "$UPGOTGETHOSTCMD" -o -z "$UPGOTTIMEOUTCMD" -o -z "$UPGOTPARALLELCMD" ]
then
	# did not have all 3 predefined, search from scratch
	UPGOTGETHOSTCMD=N
	UPGOTTIMEOUTCMD=N
	UPGOTPARALLELCMD=N
	for P in `echo $PATH | sed 's/:/ /g'`
	do
		if [ -x "$P/timeout" ]
		then
			UPGOTTIMEOUTCMD=Y
		fi
		if [ -x "$P/gethost" ]
		then
			UPGOTGETHOSTCMD=Y
		fi
		if [ -x "$P/parallel" ]
		then
			UPGOTPARALLELCMD=Y
		fi
		if [ "$UPGOTGETHOSTCMD" = Y -a "$UPGOTTIMEOUTCMD" = Y -a "$UPGOTPARALLELCMD" = Y ]
		then
			# all commands found - get out of search
			break
		fi
	done
	# give result to any potential child processes
	export UPGOTGETHOSTCMD UPGOTTIMEOUTCMD UPGOTPARALLELCMD
fi

# if looping
if [ "$LOOP" = Y ]
then
	PREV='first time'
	while :
	do
		case "$CONCURRENT" in
		Y)
			RESULT=`$0 $ARGS $* | sort`
			;;
		N)
			RESULT=`$0 $ARGS $*`
			;;
		esac
		if [ "$RESULT" != "$PREV" ]
		then
			date
			echo "$RESULT"
			PREV="$RESULT"
		fi
		sleep $SLEEPTIME
	done
	exit
fi

# if concurrent, spawn parallel processes
if [ "$CONCURRENT" = Y -a "$UPGOTPARALLELCMD" = Y -a "$#" -gt 1 ]
then
	for HP
	do
		echo "$HP"
	done | parallel $PMAX $0 $ARGS
	# note that parallel args can be passed to child, as child only has one hostname
	exit
fi

# work out which ping we have
case "$DOICMP" in
Y)
	P=`ping 2>&1 | grep .-w.*milli`
	if [ -z "$P" ]
	then
		PINGFLG="-w $TIMEOUT -c 1"
	else
		PINGFLG="-w ${TIMEOUT}000 -n 1"
	fi
	;;
esac

case "$UPGOTTIMEOUTCMD" in
Y)
	# what type of "timeout" command do we have
	case `timeout $TIMEOUT -- ls 2>&1` in
	*No\ such*)
		UPGOTTIMEOUTCMD=D
		;;
	esac
	case `timeout --help 2>&1` in
	*-s[\ ,]*)
		TIMEOUTSIG='-s2'
		;;
	*)
		TIMEOUTSIG='-2'
		;;
	esac
	;;
esac

# for each HOST:PORT given
for HP
do
	# split HOST:PORT or use default PORT values from -p option
	case "$HP" in
	*:*)
		HOST=`echo "$HP" | sed 's/:.*//'`
		PORT=`echo "$HP" | sed 's/.*://'`
		;;
	*)
		HOST="$HP"
		if [ -n "$USEPORT" ]
		then
			PORT="$USEPORT"
		else
			PORT=
		fi
		;;
	esac

	POUT=
	case "$DOICMP" in
	Y)
		# do ping
		case "$UPGOTTIMEOUTCMD" in
		Y)
			POUT=`timeout $TIMEOUTSIG $TTIMEOUT -- ping $PINGFLG $HOST 2>&1`
			;;
		D)
			POUT=`timeout $TIMEOUTSIG $TTIMEOUT ping $PINGFLG $HOST 2>&1`
			;;
		*)
			POUT=`ping $PINGFLG $HOST 2>&1`
			;;
		esac
		case "$POUT" in
		*unknown*)
			POUT='Ping=Unknown'
			;;
		*unreachable*)
			POUT='Ping=Unreachable'
			;;
		*bytes\ from*|*bytes=*|*time=*|*time\<*)
			POUT='Ping=UP'
			;;
		*)
			POUT='Ping=DOWN'
			;;
		esac
		;;
	esac

	PUP=
	if [ -n "$PORT" ]
	then
		# for each PORT, try connect
		for P in `echo $PORT | sed 's/,/ /g'`
		do
			N=`expr "$P" + 1 2>/dev/null`
			case "$N" in
			'')
				# lookup value for given service
				LPORT=`grep "^$P[ 	]*[0-9]*/tcp" /etc/services 2>/dev/null | sed 's,/tcp.*,,;s,.*[ 	],,'`
				NPORT="$P/$LPORT"
				P="$LPORT"
				;;
			*)
				# lookup name for given port
				LPORT=`grep "^[^ 	]*[ 	]*$P/tcp" /etc/services 2>/dev/null | sed 's/[ 	].*//'`
				case "$LPORT" in
				'')
					# not found
					NPORT="Port$P"
					;;
				*)
					NPORT="$LPORT/$P"
					;;
				esac
				;;
			esac
			# try the connect - use curl
			case "$UPGOTTIMEOUTCMD" in
			Y)
				COUT=`timeout $TIMEOUTSIG $TTIMEOUT -- curl -v -m$TTIMEOUT --connect-timeout $TIMEOUT $HOST:$P </dev/null 2>&1`
				;;
			D)
				COUT=`timeout $TIMEOUTSIG $TTIMEOUT curl -v -m$TTIMEOUT --connect-timeout $TIMEOUT $HOST:$P </dev/null 2>&1`
				;;
			*)
				COUT=`curl -v -m$TTIMEOUT --connect-timeout $TIMEOUT $HOST:$P </dev/null 2>&1`
				;;
			esac
			case "$COUT" in
			*resolve\ host*)
				PUP="$PUP $NPORT=Unknown"
				;;
			*Connected\ to*)
				PUP="$PUP $NPORT=UP"
				;;
			*refused*|*error:\ 111*)
				PUP="$PUP $NPORT=Refused"
				;;
			*)
				PUP="$PUP $NPORT=DOWN"
				;;
			esac
		done
	fi

	# hostname lookup
	G=
	case "$UPGOTGETHOSTCMD" in
	Y)
		case "$DODNS" in
		Y)
			G=`gethost "$HOST" 2>/dev/null | tr '\020\011' '\012\012' | sort -u | grep -v "^$HOST$"`
			;;
		esac
		;;
	esac

	echo $HOST"	"$POUT $PUP $G
done
